package mqtools;


import java.util.StringTokenizer;

/**
 * Insert the type's description here.
 * Creation date: (11/09/2001 1:37:52 PM)
 * @author: Vasant Loka
 */
public class MQOptionsAnalyzer
{
    private static final String GMO_PREFIX = "MQGMO_";
    private static final String PMO_PREFIX = "MQPMO_";
    private static final String OO_PREFIX = "MQOO_";

    private static String defaultClassName = "com.ibm.mq.MQC";
    private static Class clazz;

    /**
     * MQGMOAnalyzer constructor comment.
     */
    public MQOptionsAnalyzer()
    {
	super();

	try
	{
	    defaultClassName = "com.ibm.mq.MQC";
	    clazz = Class.forName(defaultClassName);
	}
	catch(ClassNotFoundException e)
	{
	    e.printStackTrace();
	}
    }
    /**
     * Insert the method's description here.
     * Creation date: (10/22/2001 9:42:37 AM)
     * @param: 
     * @return:
     */
    public static final String decode(int code, String optionPrefix)
    {
	optionPrefix = getOptionPrefix(optionPrefix);
	if (optionPrefix.equalsIgnoreCase("MQRC_") || optionPrefix.equalsIgnoreCase("MQCC_"))
	{
		setDefaultClassName("com.ibm.mq.MQException");
		return getField(code, optionPrefix);
	}

	setDefaultClassName("com.ibm.mq.MQC");

	String gmoString = "";
	int gmo = code;

	int codeArray = 0;
	while (gmo > 0)
	{
	    gmo /= 2;
	    codeArray++;
	}

	gmo = code;
	int value = 1;
	for (int i = 0; i < codeArray; i++)
	{
	    if ( (gmo & value) == value)
	    {
		gmoString = getField(value, optionPrefix) + " | " + gmoString;
	    }

	    value *= 2;
	}

	return gmoString.substring(0, gmoString.length() - 2);
    }
    /**
     * Insert the method's description here.
     * Creation date: (10/22/2001 9:42:37 AM)
     * @param: 
     * @return:
     */
    public static final int encode(String codeString, String optionPrefix)
    {
	optionPrefix = getOptionPrefix(optionPrefix);
	if (optionPrefix.equalsIgnoreCase("MQRC_") || optionPrefix.equalsIgnoreCase("MQCC_"))
	{
		setDefaultClassName("com.ibm.mq.MQException");
		return getValue(codeString, optionPrefix);
	}

	setDefaultClassName("com.ibm.mq.MQC");
	int gmo = 0;
	String string = null;

	String delimiters = "|";
	StringTokenizer st = new StringTokenizer(codeString, delimiters);
	while (st.hasMoreTokens() == true)
	{
	    String token = (String)st.nextToken();
	    if (token != null)
	    {
		token = token.trim();
		gmo |= getValue(token, optionPrefix);
	    }
	}

	return gmo;
    }
    /**
     * Insert the method's description here.
     * Creation date: (04/22/2002 10:21:22 AM)
     * @return java.lang.Class
     */
    public static java.lang.Class getClazz()
    {
	return getClazz(defaultClassName);
    }
    /**
     * Insert the method's description here.
     * Creation date: (04/22/2002 10:21:22 AM)
     * @return java.lang.Class
     */
    public static java.lang.Class getClazz(String className)
    {
	try
	{
	    clazz = Class.forName(className);
	}
	catch (ClassNotFoundException e)
	{
	    e.printStackTrace();
	}

	return clazz;
    }
    /**
     * Insert the method's description here.
     * Creation date: (10/22/2001 9:42:37 AM)
     * @param: 
     * @return:
     */
    private static final String getField(int code, String optionPrefix)
    {
	//System.err.println("Attempting to get field with (" + optionPrefix + ")");
	String gmo = "";
	String string = null;

	java.lang.reflect.Field[] fields = getClazz(defaultClassName).getDeclaredFields();

	for (int i = 0; i < fields.length; i++)
	{
	    try
	    {
		String fname = fields[i].getName();
		if (fname.startsWith(optionPrefix) == true)
		{
		    int value = fields[i].getInt(null);
		    if (code == value)
		    {
			gmo = fname;
			break;
		    }
		}
		else {
//			System.err.println(fname);
		}
	    }
	    catch(IllegalAccessException e)
	    {
	    }
	    catch(IllegalArgumentException e)
	    {
	    }
	}

	return gmo;
    }
    /**
     * Insert the method's description here.
     * Creation date: (10/22/2001 9:42:37 AM)
     * @param: 
     * @return:
     */
    private static final int getValue(String codeString, String optionPrefix)
    {
	    int gmo = 0;
	    String string = null;

	    java.lang.reflect.Field[] fields = getClazz(getDefaultClassName()).getDeclaredFields();

	    for (int i = 0; i < fields.length; i++)
	    {
		    try
		    {
			    String fname = fields[i].getName();
			    if (fname.startsWith(optionPrefix) == true)
			    {
				    if (codeString.equals(fname) == true)
				    {
					    int value = fields[i].getInt(null);
					    gmo = gmo | value;
					    break;
				    }
			    }
		    }
		    catch(IllegalAccessException e)
		    {
		    }
		    catch(IllegalArgumentException e)
		    {
		    }
	    }

	    return gmo;
    }
    /**
     * Insert the method's description here.
     * Creation date: (11/09/2001 1:43:02 PM)
     * @param args java.lang.String[]
     */
    public static void main(String[] args)
    {
	    String optionPrefix = MQOptionsAnalyzer.GMO_PREFIX;

	    if (args.length < 2)
	    {
		System.err.println("Usage: MQOptionAnalyzer <option prefix> [<\"message option | message option...\"> | <integer>]");
		System.err.println("Where");
		System.err.println("\toption prefix must be one of: MQOO, MQGMO, MQPMO");
		System.err.println("\tit can also be used for any other options like MQMO etc. but it has not been tested.");
		System.err.println("For example:");
		System.err.println("\tMQOptionAnalyzer MQOO MQOO_SET_IDENTITY_CONTEXT | MQOO_OUTPUT");
		System.err.println("\tMQOptionAnalyzer MQOO 1040");
		System.exit(-1);
	    }

	    optionPrefix = args[0].toUpperCase();
	    String tokens = args[1].toUpperCase();

	    try
	    {
		int code = Integer.parseInt(tokens);
		System.err.println(MQOptionsAnalyzer.decode(code, optionPrefix));
	    }
	    catch(NumberFormatException e)
	    {
		int value = MQOptionsAnalyzer.encode(tokens, optionPrefix);
		System.err.println(value);
	    }

	    //com.ibm.mq.MQC.MQMO_MATCH_MSG_SEQ_NUMBER
	    //optionPrefix = MQGMOAnalyzer.OO_PREFIX;

	    //optionPrefix = MQGMOAnalyzer.GMO_PREFIX;
	    //System.err.println(MQGMOAnalyzer.decode(24577, optionPrefix));

	    //optionPrefix = MQGMOAnalyzer.OO_PREFIX;
	    //System.err.println(MQGMOAnalyzer.decode(24577, optionPrefix));
    }
    
    private static String getOptionPrefix(String oprefix)
    {
	if (oprefix.equalsIgnoreCase("Reason Code") == true)
	{
	    return "MQRC_";
	}
	else if (oprefix.equalsIgnoreCase("MQRC") == true)
	{
	    return oprefix + "_";
	}
	else if (oprefix.equalsIgnoreCase("MQRC_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("Completion Code") == true)
	{
	    return "MQCC_";
	}
	else if (oprefix.equalsIgnoreCase("MQCC") == true)
	{
	    return oprefix + "_";
	}
	else if (oprefix.equalsIgnoreCase("MQCC_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("Get Options") == true)
	{
	    return "MQGMO_";
	}
	else if (oprefix.equalsIgnoreCase("MQGMO") == true)
	{
	    return oprefix + "_";
	}
	else if (oprefix.equalsIgnoreCase("MQGMO_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("Put Options") == true)
	{
	    return "MQPMO_";
	}
	else if (oprefix.equalsIgnoreCase("MQPMO_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("MQPMO") == true)
	{
	    return oprefix + "_";
	}
	else if (oprefix.equalsIgnoreCase("Open Options") == true)
	{
	    return "MQOO_";
	}
	else if (oprefix.equalsIgnoreCase("MQOO_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("MQOO") == true)
	{
	    return oprefix + "_";
	}
	else if (oprefix.equalsIgnoreCase("Match Options") == true)
	{
	    return "MQMO_";
	}
	else if (oprefix.equalsIgnoreCase("MQMO_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("MQMO") == true)
	{
	    return oprefix + "_";
	}
/*
	else if (oprefix.equalsIgnoreCase("Feedback Code") == true)
	{
	    return "MQFB_";
	}
	else if (oprefix.equalsIgnoreCase("MQFB_") == true)
	{
	    return oprefix;
	}
	else if (oprefix.equalsIgnoreCase("MQFB") == true)
	{
	    return oprefix + "_";
	}
*/
	else
	{
	    throw new IllegalArgumentException("Invalid option " + oprefix);
	}
    }
    
/**
 * Insert the method's description here.
 * Creation date: (5/9/2002 5:46:47 AM)
 * @return java.lang.String
 */
public static java.lang.String getDefaultClassName() {
	return defaultClassName;
}

/**
 * Insert the method's description here.
 * Creation date: (5/9/2002 5:46:47 AM)
 * @param newDefaultClassName java.lang.String
 */
public static void setDefaultClassName(java.lang.String newDefaultClassName) {
	defaultClassName = newDefaultClassName;
}
}
